// Package database ...
//
// Description : mysql ...
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 2022-05-15 19:32
package database

import (
	"fmt"

	"gorm.io/driver/mysql"

	"gorm.io/gorm"
)

// SystemDao 执行一些系统的mysql命令
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 19:32 2022/5/15
type SystemDao struct {
}

// Connect 连接数据库
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 17:58 2023/4/9
func (sd *SystemDao) Connect(host string, port string, user string, password string, database string) (*gorm.DB, error) {
	// 	db, err := gorm.Open("mysql", "root:root@(localhost)/gorm-test?charset=utf8mb4&parseTime=True&loc=Local")
	conn := fmt.Sprintf("%v:%v@(%v:%v)/%v?charset=utf8mb4&parseTime=True&loc=Local", user, password, host, port, database)
	return gorm.Open(mysql.Open(conn), &gorm.Config{})
}

// SelectDatabase 切换数据库
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 17:44 2023/4/9
func (sd *SystemDao) SelectDatabase(dbInstance *gorm.DB, database string) error {
	return dbInstance.Raw("USE " + database).Error
}

// GetDatabaseList 获取数据库列表
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 17:46 2023/4/9
func (sd *SystemDao) GetDatabaseList(dbInstance *gorm.DB) ([]string, error) {
	var (
		err    error
		result []string
	)

	if err = dbInstance.Raw("SHOW DATABASES").Scan(&result).Error; nil != err {
		return nil, err
	}
	return result, nil
}

// GetTableList 查询数据表
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 19:36 2022/5/15
func (sd *SystemDao) GetTableList(dbInstance *gorm.DB) ([]string, error) {
	var (
		err    error
		result []string
	)

	if err = dbInstance.Raw("SHOW TABLES").Scan(&result).Error; nil != err {
		return nil, err
	}
	return result, nil
}

// GetCreateTableSQL 获取建表SQL
//
// Author : go_developer@163.com<白茶清欢>
//
// Date : 20:58 2022/5/15
func (sd *SystemDao) GetCreateTableSQL(dbInstance *gorm.DB, table string) (string, error) {
	var (
		err    error
		result map[string]interface{}
	)

	if err = dbInstance.Raw("SHOW CREATE TABLE `" + table + "`").Scan(&result).Error; nil != err {
		return "", err
	}
	return fmt.Sprintf("%v", result["Create Table"]), nil
}
